@@ -57,7 +57,7 @@ class LupusecAPI:
     """Interface to Lupusec Webservices."""
 
     def __init__(self, username, password, ip_address) -> None:
-        """LupusecAPI constructor to interface Lupusec Alarm System."""
+        """LupusecAPI constructor to interface Lupusec Alarm System XT2Plus."""
         self._username = username
         self._password = password
         self._ip_address = ip_address
@@ -115,7 +115,6 @@ class LupusecAPI:
         try:
             async with session.get(url, ssl=False) as resp:
                 _LOGGER.debug("Response_Status=%s", resp.status)
-                _LOGGER.debug("Content_Type=%s", resp.headers["content-type"])
 
                 # check for Response Status other than 200
                 if resp.status != 200:
@@ -399,7 +398,7 @@ class LupusecAPI:
         # ToDo: update frequency shall not be hard-coded -> transfer to constant
         if self._cacheBinarySensors is None or timeNow - self._cacheStampP > 2.0:
             _LOGGER.debug("...BinarySensors need update from Lupusec System.")
-            self._cacheStamp_p = timeNow
+            self._cacheStampP = timeNow
 
             async with aiohttp.ClientSession(auth=self._auth) as session:
                 _LOGGER.debug("auth.encode: %s", self._auth.encode())  
@@ -415,8 +414,8 @@ class LupusecAPI:
                     if (len(device_content) != 0):             
                         binary_sensors = []
                         for device in device_content:
-                            print("sid: ", device["sid"], ", name: ", device["name"], 
-                                ", type: ", device["type"], ", status: ", device["status"])
+                            _LOGGER.debug("BinarySensor found: sid=%s, name=%s, type=%s, status=%s", 
+                                device["sid"], device["name"], device["type"], device["status_ex"] )
                             if (device["type"] in CONST.TYPES_BIN_SENSOR):    
                                 binary_sensors.append(device)
                                 _LOGGER.debug("device is BinarySensor...added.")
@@ -431,7 +430,8 @@ class LupusecAPI:
         _LOGGER.debug("__init__.py.get_binary_sensors() finished.") 
 
         if (len(self._cacheBinarySensors) != 0):
-            print("Number of switches=", len(self._cacheBinarySensors))            
+            _LOGGER.debug("Number of BinarySensors=%s", len(self._cacheBinarySensors)) 
+            print("Number of BinarySensors=", len(self._cacheBinarySensors))            
             for sensor in self._cacheBinarySensors:
                 print("sid: ", sensor["sid"], 
                     ", name: ", sensor["name"], 
@@ -442,6 +442,129 @@ class LupusecAPI:
         return self._cacheBinarySensors
 
 
+
+    async def get_device_async(self, device_id) -> Dict:
+        """Async method to get a single device."""
+        _LOGGER.debug("__init__.py.get_device_async() called for device_id=%s", device_id)
+        timeNow = time.time()
+        
+        # Get device from cache or update from Lupusec System
+        # ToDo: update frequency shall not be hard-coded -> transfer to constant
+        if self._cacheBinarySensors is None or timeNow - self._cacheStampP > 10.0:
+            # time elapsed, devices need update from Lupusec System.
+            if self._cacheBinarySensors is None:
+                _LOGGER.debug("__init__.py.get_device_async(): cacheBinarySensors is none, update from Lupusec System started.")
+            if timeNow - self._cacheStampP > 2.0:
+                _LOGGER.debug("__init__.py.get_device_async(): time elapsed, update from Lupusec System started.")    
+            self._cacheStampP = timeNow
+
+            async with aiohttp.ClientSession(auth=self._auth) as session:
+                _LOGGER.debug("auth.encode: %s", self._auth.encode())  
+
+                # Get all devices from LupusecAPI
+                _LOGGER.debug("__init__.py.get_device_async(): REQUEST=%s", CONST.DEVICE_LIST_REQUEST)
+                get_bin_sensors_response = await LupusecAPI._async_api_call(self._ip_address, session, CONST.DEVICE_LIST_REQUEST)
+                _LOGGER.debug("_async_api_call(): done. check response...")
+                # Retreive Device Liste from Response
+                if CONST.DEVICE_LIST_HEADER in get_bin_sensors_response:
+                    device_content = get_bin_sensors_response[CONST.DEVICE_LIST_HEADER]
+                    _LOGGER.debug("Number of devices found: %s", len(device_content))    
+                    if (len(device_content) != 0):             
+                        binary_sensors = []
+                        for device in device_content:
+                            if (device["type"] in CONST.TYPES_BIN_SENSOR):    
+                                binary_sensors.append(device)
+                                _LOGGER.debug("BinarySensor found: sid=%s, name=%s, type=%s, status=%s", 
+                                    device["sid"], device["name"], device["type"], device["status_ex"] )
+                                _LOGGER.debug("sid=%s, python_class=%s", 
+                                    device["sid"], type(device["sid"]))                                    
+                            else :
+                                _LOGGER.debug("device is no BinarySensor...skipping.")
+                        self._cacheBinarySensors = binary_sensors
+                        _LOGGER.debug("get_device_async(): No of updated BinarySensors: %s", len(self._cacheBinarySensors))
+                    else : 
+                        _LOGGER.info("ERROR: get_device_async(): no BinarySensors found.")
+                else :
+                    _LOGGER.info("ERROR: get_device_async(): no BinarySensors found.")
+            _LOGGER.debug("__init__.py.get_device_async(): update from LupusecAPI finished.") 
+
+        # time not elapsed, Devices do not need update from Lupusec System...using Cache
+        _LOGGER.debug("__init__.py.get_device_async(): using Cache to provide Update on BinarySensor.")
+        if (len(self._cacheBinarySensors) != 0):  
+            _LOGGER.debug("__init__.py.get_device_async(): Number of BinarySensors=%s", len(self._cacheBinarySensors))             
+            for sensor in self._cacheBinarySensors:
+                #_LOGGER.debug("__init__.py.get_device_async(): device with sid=%s, name=%s", sensor["sid"], sensor["name"])
+                if (sensor["sid"] == device_id):
+                    _LOGGER.debug("__init__.py.get_device_async(): matching device with sid=%s found: status=%s", sensor["sid"], sensor["status_ex"])
+                    return sensor    
+            _LOGGER.error("__init__.py.get_device_async(): %s BinarySensors checked, but no device found with sid=%s", len(self._cacheBinarySensors), device_id)
+            return {}
+        else :
+            _LOGGER.debug("ERROR: length of self._cacheBinarySensors = 0") 
+            return {}
+
+
+    async def get_switch_async(self, device_id) -> Dict:
+        """Async method to get a single switch device."""
+        _LOGGER.debug("__init__.py.get_switch_async() called for device_id=%s", device_id)
+        timeNow = time.time()
+        
+        # Get device from cache or update from Lupusec System
+        # ToDo: update frequency shall not be hard-coded -> transfer to constant
+        if self._cacheSwitches is None or timeNow - self._cacheStampP > 10.0:
+            # time elapsed, devices need update from Lupusec System.
+            if self._cacheSwitches is None:
+                _LOGGER.debug("__init__.py.get_switch_async(): cacheSwitchess is none, update from Lupusec System started.")
+            if timeNow - self._cacheStampP > 2.0:
+                _LOGGER.debug("__init__.py.get_switch_async(): time elapsed, update from Lupusec System started.")    
+            self._cacheStampP = timeNow
+
+            async with aiohttp.ClientSession(auth=self._auth) as session:
+                _LOGGER.debug("auth.encode: %s", self._auth.encode())  
+
+                # Get all devices from LupusecAPI
+                _LOGGER.debug("__init__.py.get_switch_async(): REQUEST=%s", CONST.DEVICE_LIST_REQUEST)
+                get_bin_switches_response = await LupusecAPI._async_api_call(self._ip_address, session, CONST.DEVICE_LIST_REQUEST)
+                _LOGGER.debug("_async_api_call(): done. check response...")
+                # Retreive Device Liste from Response
+                if CONST.DEVICE_LIST_HEADER in get_bin_switches_response:
+                    device_content = get_bin_switches_response[CONST.DEVICE_LIST_HEADER]
+                    _LOGGER.debug("Number of devices found: %s", len(device_content))    
+                    if (len(device_content) != 0):             
+                        switches = []
+                        for device in device_content:
+                            if (device["type"] in CONST.TYPES_SWITCH):    
+                                switches.append(device)
+                                _LOGGER.debug("Switch found: sid=%s, name=%s, type=%s, status=%s", 
+                                    device["sid"], device["name"], device["type"], device["status_ex"] )
+                                _LOGGER.debug("sid=%s, python_class=%s", 
+                                    device["sid"], type(device["sid"]))                                    
+                            else :
+                                _LOGGER.debug("device is no Switch...skipping.")
+                        self._cacheSwitches = switches
+                        _LOGGER.debug("get_switch_async(): No of updated switches: %s", len(self._cacheSwitches))
+                    else : 
+                        _LOGGER.info("ERROR: get_switch_async(): no Switch found.")
+                else :
+                    _LOGGER.info("ERROR: get_switch_async(): no Switch found.")
+            _LOGGER.debug("__init__.py.get_switch_async(): update from LupusecAPI finished.") 
+
+        # time not elapsed, Devices do not need update from Lupusec System...using Cache
+        _LOGGER.debug("__init__.py.get_switch_async(): using Cache to provide Update on Switch.")
+        if (len(self._cacheSwitches) != 0):  
+            _LOGGER.debug("__init__.py.get_switch_async(): Number of Switches=%s", len(self._cacheSwitches))             
+            for switch in self._cacheSwitches:
+                #_LOGGER.debug("__init__.py.get_switch_async(): device with sid=%s, name=%s", sensor["sid"], sensor["name"])
+                if (switch["sid"] == device_id):
+                    _LOGGER.debug("__init__.py.get_switch_async(): matching device with sid=%s found: status=%s", switch["sid"], switch["status_ex"])
+                    return switch    
+            _LOGGER.error("__init__.py.get_switch_async(): %s Switches checked, but no device found with sid=%s", len(self._cacheSwitches), device_id)
+            return {}
+        else :
+            _LOGGER.debug("ERROR: length of self._cacheSwitches = 0") 
+            return {}
+
+
     def get_sensors(self):
         _LOGGER.debug("get_sensors() called:")
         stamp_now = time.time()
